## -----------------------------------------------------------------------------------------------------------
## Code corresponding to: Forecasting Generalized Quantiles of Electricity Demand: A Functional Data Approach
## Loads and processes Data of the BU Stadtwerke Saarbruecken
## -----------------------------------------------------------------------------------------------------------

Load          = read.delim("Data\\LoadBU.txt", header=F)[,c(1,2,8)]
Temperature   = read.delim("Data\\TempBU.txt", header=F)[,7]
Sun           = read.delim("Data\\SunBU.txt", header=F)
Load_mat      = reshape(Load,timevar="V2", idvar="V1", direction="wide")
Load_mat[,1]  = as.Date(Load_mat[,1])

Load_mat      = Load_mat[-816,-98]

remove        = c(seq.Date(from=as.Date("2010-01-01"),to=as.Date("2010-01-05"),by="day"),
                  seq.Date(from=as.Date("2010-12-23"),to=as.Date("2011-01-05"),by="day"),
                  seq.Date(from=as.Date("2011-12-23"),to=as.Date("2012-01-05"),by="day"),
                  seq.Date(from=as.Date("2012-12-23"),to=as.Date("2012-12-31"),by="day"))

Temperature   = Temperature[-c(which(Load_mat[,1]%in%remove))]
Sun           = Sun[-c(which(Load_mat[,1]%in%remove)),2]
Load_mat      = Load_mat[-c(which(Load_mat[,1]%in%remove)),] 

for(i in 2:97){
  a             = which(is.na(Load_mat[,i]))
  weekm         = a-rep(7,length(a))
  weekp         = a+rep(7,length(a))
  Load_mat[a,i] = (Load_mat[weekm,i]+Load_mat[weekp,i])/2
}

dummy         = cbind(as.numeric(Load_mat[,1] %in% as.Date(c("2010-04-02","2010-04-05","2010-05-01","2010-05-13","2010-05-24",
                "2010-06-03","2010-10-03","2010-08-15","2010-11-01","2011-04-22","2011-04-25","2011-05-01","2011-06-02","2011-06-13",
                "2011-06-23","2011-08-15","2011-10-03","2011-11-01","2012-04-06","2012-04-09","2012-05-01","2012-05-17","2012-05-28",
                "2012-06-07","2012-10-03","2012-11-01", "2013-03-29","2013-04-01","2013-05-01","2013-05-09","2013-05-20","2013-05-30"))),
                as.numeric(Load_mat[,1] %in% as.Date(c("2011-06-03","2011-06-24","2012-04-30","2012-06-08"))),
                as.numeric(Load_mat[,1] %in% as.Date(c("2010-10-03","2010-08-15","2011-05-01"))),
                as.numeric(Load_mat[,1] %in% as.Date(c("2010-05-01"))))

rm(list=ls()[! ls() %in% c("Load","Load_mat","Temperature","Sun","Load_pred","dummy")])