## -----------------------------------------------------------------------------------------------------------
## Code corresponding to: Forecasting Generalized Quantiles of Electricity Demand: A Functional Data Approach
## Computes Forecasts of BU Saarbruecken using triple seasonal Holt-Winter (TSHW) model
## -----------------------------------------------------------------------------------------------------------

rm(list = ls())
graphics.off()
# setwd("...\\Data and Command Files") # Set working directory

libraries<-c("forecast")
lapply(libraries,function(x)if(!(x %in% installed.packages())){install.packages(x)})
lapply(libraries,require,quietly=TRUE,character.only=TRUE)

source("Code\\Functions.R")

## Load Data
Load          = read.delim("Data\\LoadBU.txt", header=F)[,c(1,2,8)]

Load_mat      = reshape(Load,timevar="V2", idvar="V1", direction="wide")
Load_mat[,1]  = as.Date(Load_mat[,1])
Load_mat      = Load_mat[-816,-98]

remove        = c(seq.Date(from=as.Date("2010-01-01"),to=as.Date("2010-01-05"),by="day"),
              seq.Date(from=as.Date("2010-12-23"),to=as.Date("2011-01-05"),by="day"),
              seq.Date(from=as.Date("2011-12-23"),to=as.Date("2012-01-05"),by="day"),
              seq.Date(from=as.Date("2012-12-23"),to=as.Date("2012-12-31"),by="day"))

PH            = c(as.Date(c("2010-04-02","2010-04-05","2010-05-01","2010-05-13","2010-05-24","2010-06-03","2010-08-15","2010-10-03","2010-11-01")),
              as.Date(c("2011-04-22","2011-04-25","2011-05-01","2011-06-02","2011-06-13","2011-06-23","2011-08-15","2011-10-03","2011-11-01")),
              as.Date(c("2012-04-06","2012-04-09","2012-05-01","2012-05-17","2012-05-25","2012-06-07","2012-08-15","2012-10-03","2012-11-01")))

Load_mat[which(Load_mat[,1]%in%PH),2:97] = NA

for(i in 2:97){
  a = which(is.na(Load_mat[,i]))
  weekm = a-rep(7,length(a))
  weekp = a+rep(7,length(a))
  Load_mat[a,i] = (Load_mat[weekm,i]+Load_mat[weekp,i])/2
}

Load_mat = Load_mat[-c(which(Load_mat[,1]%in%remove)),]   

## Forecast TSHW using tbats
Load_list = unlist(as.data.frame(t(Load_mat[,-1])))
n = 702
h = 350
p = 2
hours = 96
weekhours = 7*96
yearhours = 351.25*96
forecast_TSHW = matrix(ncol=hours,nrow=351)
idx.eval = (1:96)+48*(p-1)

for(k in 0:h){
  insample = Load_list[1:(n*96+k*96+(p-1)*48)]
  hw = tbats(insample,use.trend=FALSE,use.box.cox=FALSE,seasonal.periods=c(hours,weekhours,yearhours),use.arma.errors=TRUE,use.parallel=TRUE)
  forecast_TSHW[(k+p),] = forecast.tbats(hw,h=(96+(p-1)*48))$mean[idx.eval]
  print(k)
}

## Evaluate Results
Eval_TSHW = EVAL(forecast_TSHW, Load_mat[,-1], h = h, n = n, p = p)
MSE_TSHW  = mean(Eval_TSHW[[1]], na.rm=TRUE)
MAPE_TSHW = mean(Eval_TSHW[[2]], na.rm=TRUE)
